@@ -9,8 +9,8 @@ if ! qlist -qI dev-libs/gnulib > /dev/null ; then
 	echo "please install dev-libs/gnulib"
 	exit 1
 fi
-if ! qlist -qI sys-devel/autoconf-archive > /dev/null ; then
-	echo "please install sys-devel/autoconf-archive"
+if ! qlist -qI dev-build/autoconf-archive > /dev/null ; then
+	echo "please install dev-build/autoconf-archive"
 	exit 1
 fi
 
@@ -28,6 +28,10 @@ done
 # reload the gnulib code
 PATH=/usr/local/src/gnu/gnulib:${PATH}
 mods="
+	crypto/md5-buffer
+	crypto/sha1-buffer
+	crypto/sha256-buffer
+	crypto/sha512-buffer
 	dirent
 	faccessat
 	fdopendir
@@ -57,5 +61,24 @@ v gnulib-tool \
 	--no-vc-files \
 	${mods}
 
+{
+	sed -e '/^# BEGIN GNULIB/,/^# END GNULIB/d' .gitignore
+	cat <<- EOM
+		# BEGIN GNULIB  --  keep this at the end of this file
+		# regenerate using:
+		#   ls autotools/gnulib/*.in.h | sed -e 's/\.in\.h/.h/' | sed -e 's:_:/:'
+		# or use autogen.sh
+	EOM
+	ls autotools/gnulib/*.in.h | sed -e 's/\.in\.h/.h/' | sed -e 's:_:/:'
+	cat <<- EOM
+		# manual additions
+		autotools/gnulib/sys
+		autotools/gnulib/malloc/scratch_buffer.gl.h
+		*.dirstamp
+		# END GNULIB
+	EOM
+} > .gitignore.new
+[[ -s .gitignore.new ]] && mv .gitignore.new .gitignore
+
 export AUTOMAKE="automake --foreign"
 v autoreconf -i -f
