@@ -1,5 +1,5 @@
 /*
- * Copyright 2018-2020 Gentoo Foundation
+ * Copyright 2018-2024 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2018-     Fabian Groffen  - <grobian@gentoo.org>
@@ -12,25 +12,22 @@
 
 #include "main.h"
 
-#ifdef HAVE_SSL
-# include <openssl/md5.h>
-# include <openssl/sha.h>
-# include <openssl/whrlpool.h>
-#else
-# include "hash_md5_sha1.h"
-#endif
 #ifdef HAVE_BLAKE2B
 # include <blake2.h>
 #endif
 
+#include "md5.h"
+#include "sha1.h"
+#include "sha256.h"
+#include "sha512.h"
+
 #include "hash.h"
 
 void
 hash_hex(char *out, const unsigned char *buf, const int length)
 {
 	switch (length) {
-		/* MD5_DIGEST_LENGTH */
-		case 16:
+		case 16: /* MD5_DIGEST_SIZE */
 			snprintf(out, 32 + 1,
 					"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
 					"%02x%02x%02x%02x%02x%02x",
@@ -40,8 +37,7 @@ hash_hex(char *out, const unsigned char *buf, const int length)
 					buf[15]
 					);
 			break;
-		/* SHA1_DIGEST_LENGTH */
-		case 20:
+		case 20: /* SHA1_DIGEST_SIZE */
 			snprintf(out, 40 + 1,
 					"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
 					"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
@@ -51,8 +47,7 @@ hash_hex(char *out, const unsigned char *buf, const int length)
 					buf[15], buf[16], buf[17], buf[18], buf[19]
 					);
 			break;
-		/* SHA256_DIGEST_LENGTH */
-		case 32:
+		case 32: /* SHA256_DIGEST_SIZE */
 			snprintf(out, 64 + 1,
 					"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
 					"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
@@ -67,8 +62,7 @@ hash_hex(char *out, const unsigned char *buf, const int length)
 					buf[30], buf[31]
 					);
 			break;
-		/* SHA512_DIGEST_LENGTH, WHIRLPOOL_DIGEST_LENGTH, BLAKE2B_OUTBYTES */
-		case 64:
+		case 64: /* SHA512_DIGEST_SIZE, BLAKE2B_OUTBYTES */
 			snprintf(out, 128 + 1,
 					"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
 					"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
@@ -104,6 +98,157 @@ hash_hex(char *out, const unsigned char *buf, const int length)
 	}
 }
 
+/* len func(dest,destlen,cbctx) */
+typedef size_t (*read_cb)(char *,size_t,void *);
+
+static size_t read_stdio(char *dest, size_t destlen, void *ctx)
+{
+	FILE *io = ctx;
+
+	return fread(dest, 1, destlen, io);
+}
+
+struct bufctx {
+	const char *buf;
+	size_t      buflen;
+};
+
+static size_t read_buffer(char *dest, size_t destlen, void *ctx)
+{
+	struct bufctx *membuf = ctx;
+	size_t         readlen;
+
+	readlen = destlen;
+	if (readlen > membuf->buflen)
+		readlen = membuf->buflen;
+
+	memcpy(dest, membuf->buf, readlen);
+
+	/* update buffer to the remainder */
+	membuf->buf    += readlen;
+	membuf->buflen -= readlen;
+
+	return readlen;
+}
+
+static int
+hash_multiple_internal(
+		read_cb rcb,
+		void   *ctx,
+		char   *md5,
+		char   *sha1,
+		char   *sha256,
+		char   *sha512,
+		char   *blak2b,
+		size_t *flen,
+		int     hashes)
+{
+	size_t            len;
+	char              data[8192];
+
+	struct md5_ctx    m5;
+	struct sha1_ctx   s1;
+	struct sha256_ctx s256;
+	struct sha512_ctx s512;
+#ifdef HAVE_BLAKE2B
+	blake2b_state     bl2b;
+#else
+	(void)blak2b;
+#endif
+
+	*flen = 0;
+
+	md5_init_ctx(&m5);
+	sha1_init_ctx(&s1);
+	sha256_init_ctx(&s256);
+	sha512_init_ctx(&s512);
+#ifdef HAVE_BLAKE2B
+	blake2b_init(&bl2b, BLAKE2B_OUTBYTES);
+#endif
+
+	while ((len = rcb(data, sizeof(data), ctx)) > 0) {
+		*flen += len;
+#pragma omp parallel sections
+		{
+#pragma omp section
+			{
+				if (hashes & HASH_MD5)
+					md5_process_bytes(data, len, &m5);
+			}
+#pragma omp section
+			{
+				if (hashes & HASH_SHA1)
+					sha1_process_bytes(data, len, &s1);
+			}
+#pragma omp section
+			{
+				if (hashes & HASH_SHA256)
+					sha256_process_bytes(data, len, &s256);
+			}
+#pragma omp section
+			{
+				if (hashes & HASH_SHA512)
+					sha512_process_bytes(data, len, &s512);
+			}
+#ifdef HAVE_BLAKE2B
+#pragma omp section
+			{
+				if (hashes & HASH_BLAKE2B)
+					blake2b_update(&bl2b, (unsigned char *)data, len);
+			}
+#endif
+		}
+	}
+
+#pragma omp parallel sections
+	{
+#pragma omp section
+		{
+			if (hashes & HASH_MD5) {
+				unsigned char md5buf[MD5_DIGEST_SIZE];
+				md5_finish_ctx(&m5, md5buf);
+				hash_hex(md5, md5buf, MD5_DIGEST_SIZE);
+			}
+		}
+#pragma omp section
+		{
+			if (hashes & HASH_SHA1) {
+				unsigned char sha1buf[SHA1_DIGEST_SIZE];
+				sha1_finish_ctx(&s1, sha1buf);
+				hash_hex(sha1, sha1buf, SHA1_DIGEST_SIZE);
+			}
+		}
+#pragma omp section
+		{
+			if (hashes & HASH_SHA256) {
+				unsigned char sha256buf[SHA256_DIGEST_SIZE];
+				sha256_finish_ctx(&s256, sha256buf);
+				hash_hex(sha256, sha256buf, SHA256_DIGEST_SIZE);
+			}
+		}
+#pragma omp section
+		{
+			if (hashes & HASH_SHA512) {
+				unsigned char sha512buf[SHA512_DIGEST_SIZE];
+				sha512_finish_ctx(&s512, sha512buf);
+				hash_hex(sha512, sha512buf, SHA512_DIGEST_SIZE);
+			}
+		}
+#ifdef HAVE_BLAKE2B
+#pragma omp section
+		{
+			if (hashes & HASH_BLAKE2B) {
+				unsigned char blak2bbuf[BLAKE2B_OUTBYTES];
+				blake2b_final(&bl2b, blak2bbuf, BLAKE2B_OUTBYTES);
+				hash_hex(blak2b, blak2bbuf, BLAKE2B_OUTBYTES);
+			}
+		}
+#endif
+	}
+
+	return 0;
+}
+
 /**
  * Computes the hashes for file fname and writes the hex-representation
  * for those hashes into the address space pointed to by the return
@@ -120,178 +265,23 @@ hash_multiple_file_fd(
 		char *sha1,
 		char *sha256,
 		char *sha512,
-		char *whrlpl,
 		char *blak2b,
 		size_t *flen,
 		int hashes)
 {
 	FILE *f;
-	char data[8192];
-	size_t len;
-#ifdef HAVE_SSL
-	MD5_CTX m5;
-	SHA_CTX s1;
-	SHA256_CTX s256;
-	SHA512_CTX s512;
-	WHIRLPOOL_CTX whrl;
-#else
-	struct md5_ctx_t m5;
-	struct sha1_ctx_t s1;
-	(void)sha256;
-	(void)sha512;
-	(void)whrlpl;
-#endif
-#ifdef HAVE_BLAKE2B
-	blake2b_state bl2b;
-#else
-	(void)blak2b;
-#endif
+	int   ret;
 
-	*flen = 0;
 	if ((f = fdopen(fd, "r")) == NULL)
 		return -1;
 
-#ifdef HAVE_SSL
-	MD5_Init(&m5);
-	SHA1_Init(&s1);
-	SHA256_Init(&s256);
-	SHA512_Init(&s512);
-	WHIRLPOOL_Init(&whrl);
-#else
-	md5_begin(&m5);
-	sha1_begin(&s1);
-#endif
-#ifdef HAVE_BLAKE2B
-	blake2b_init(&bl2b, BLAKE2B_OUTBYTES);
-#endif
+	ret = hash_multiple_internal(read_stdio, f,
+								 md5, sha1, sha256, sha512, blak2b,
+								 flen, hashes);
 
-	while ((len = fread(data, 1, sizeof(data), f)) > 0) {
-		*flen += len;
-#pragma omp parallel sections
-		{
-#ifdef HAVE_SSL
-#pragma omp section
-			{
-				if (hashes & HASH_MD5)
-					MD5_Update(&m5, data, len);
-			}
-#pragma omp section
-			{
-				if (hashes & HASH_SHA1)
-					SHA1_Update(&s1, data, len);
-			}
-#pragma omp section
-			{
-				if (hashes & HASH_SHA256)
-					SHA256_Update(&s256, data, len);
-			}
-#pragma omp section
-			{
-				if (hashes & HASH_SHA512)
-					SHA512_Update(&s512, data, len);
-			}
-#pragma omp section
-			{
-				if (hashes & HASH_WHIRLPOOL)
-					WHIRLPOOL_Update(&whrl, data, len);
-			}
-#else
-#pragma omp section
-			{
-				if (hashes & HASH_MD5)
-					md5_hash(data, len, &m5);
-			}
-#pragma omp section
-			{
-				if (hashes & HASH_SHA1)
-					sha1_hash(data, len, &s1);
-			}
-#endif
-#ifdef HAVE_BLAKE2B
-#pragma omp section
-			{
-				if (hashes & HASH_BLAKE2B)
-					blake2b_update(&bl2b, (unsigned char *)data, len);
-			}
-#endif
-		}
-	}
 	fclose(f);
 
-#pragma omp parallel sections
-	{
-#ifdef HAVE_SSL
-#pragma omp section
-		{
-			if (hashes & HASH_MD5) {
-				unsigned char md5buf[MD5_DIGEST_LENGTH];
-				MD5_Final(md5buf, &m5);
-				hash_hex(md5, md5buf, MD5_DIGEST_LENGTH);
-			}
-		}
-#pragma omp section
-		{
-			if (hashes & HASH_SHA1) {
-				unsigned char sha1buf[SHA_DIGEST_LENGTH];
-				SHA1_Final(sha1buf, &s1);
-				hash_hex(sha1, sha1buf, SHA_DIGEST_LENGTH);
-			}
-		}
-#pragma omp section
-		{
-			if (hashes & HASH_SHA256) {
-				unsigned char sha256buf[SHA256_DIGEST_LENGTH];
-				SHA256_Final(sha256buf, &s256);
-				hash_hex(sha256, sha256buf, SHA256_DIGEST_LENGTH);
-			}
-		}
-#pragma omp section
-		{
-			if (hashes & HASH_SHA512) {
-				unsigned char sha512buf[SHA512_DIGEST_LENGTH];
-				SHA512_Final(sha512buf, &s512);
-				hash_hex(sha512, sha512buf, SHA512_DIGEST_LENGTH);
-			}
-		}
-#pragma omp section
-		{
-			if (hashes & HASH_WHIRLPOOL) {
-				unsigned char whrlplbuf[WHIRLPOOL_DIGEST_LENGTH];
-				WHIRLPOOL_Final(whrlplbuf, &whrl);
-				hash_hex(whrlpl, whrlplbuf, WHIRLPOOL_DIGEST_LENGTH);
-			}
-		}
-#else
-#pragma omp section
-		{
-			if (hashes & HASH_MD5) {
-				unsigned char md5buf[16];
-				md5_end(md5buf, &m5);
-				hash_hex(md5, md5buf, 16);
-			}
-		}
-#pragma omp section
-		{
-			if (hashes & HASH_SHA1) {
-				unsigned char sha1buf[20];
-				sha1_end(sha1buf, &s1);
-				hash_hex(sha1, sha1buf, 20);
-			}
-		}
-#endif
-#ifdef HAVE_BLAKE2B
-#pragma omp section
-		{
-			if (hashes & HASH_BLAKE2B) {
-				unsigned char blak2bbuf[BLAKE2B_OUTBYTES];
-				blake2b_final(&bl2b, blak2bbuf, BLAKE2B_OUTBYTES);
-				hash_hex(blak2b, blak2bbuf, BLAKE2B_OUTBYTES);
-			}
-		}
-#endif
-	}
-
-	return 0;
+	return ret;
 }
 
 int
@@ -303,7 +293,6 @@ hash_multiple_file_at_cb(
 		char *sha1,
 		char *sha256,
 		char *sha512,
-		char *whrlpl,
 		char *blak2b,
 		size_t *flen,
 		int hashes)
@@ -321,7 +310,7 @@ hash_multiple_file_at_cb(
 	}
 
 	ret = hash_multiple_file_fd(fd, md5, sha1, sha256, sha512,
-			whrlpl, blak2b, flen, hashes);
+			blak2b, flen, hashes);
 
 	if (ret != 0)
 		close(fd);
@@ -340,11 +329,10 @@ hash_file_at_cb(int pfd, const char *fname, int hash, hash_cb_t cb)
 		case HASH_SHA1:
 		case HASH_SHA256:
 		case HASH_SHA512:
-		case HASH_WHIRLPOOL:
 		case HASH_BLAKE2B:
 			if (hash_multiple_file_at_cb(pfd, fname, cb,
 					_hash_file_buf, _hash_file_buf, _hash_file_buf,
-					_hash_file_buf, _hash_file_buf, _hash_file_buf,
+					_hash_file_buf, _hash_file_buf,
 					&dummy, hash) != 0)
 				return NULL;
 			break;
@@ -354,3 +342,35 @@ hash_file_at_cb(int pfd, const char *fname, int hash, hash_cb_t cb)
 
 	return _hash_file_buf;
 }
+
+char *
+hash_string(const char *buf, ssize_t buflen, int hash)
+{
+	struct bufctx membuf;
+	size_t        dummy;
+
+	if (buflen < 0)
+		buflen = (ssize_t)strlen(buf);
+
+	membuf.buf    =  buf;
+	membuf.buflen = (size_t)buflen;
+
+	switch (hash) {
+		case HASH_MD5:
+		case HASH_SHA1:
+		case HASH_SHA256:
+		case HASH_SHA512:
+		case HASH_BLAKE2B:
+			if (hash_multiple_internal(read_buffer, &membuf,
+									   _hash_file_buf, _hash_file_buf,
+									   _hash_file_buf, _hash_file_buf,
+									   _hash_file_buf,
+									   &dummy, hash) != 0)
+				return NULL;
+			break;
+		default:
+			return NULL;
+	}
+
+	return _hash_file_buf;
+}
