@@ -1,20 +1,27 @@
 /*
- * Copyright 2005-2018 Gentoo Foundation
+ * Copyright 2005-2021 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
  *
  * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
  * Copyright 2005-2014 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2018-     Fabian Groffen  - <grobian@gentoo.org>
  */
 
-#if !defined(HAVE_SCANDIRAT)
-# if defined(__GLIBC__) && (__GLIBC__ << 8 | __GLIBC_MINOR__) > (2 << 8 | 14)
-#  define HAVE_SCANDIRAT
-# endif
-#endif
+#include "main.h"
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <dirent.h>
+#include <stdlib.h>
+#include <string.h>
+#include <xalloc.h>
+
+#include "scandirat.h"
 
 #if !defined(HAVE_SCANDIRAT)
 
-static int
+int
 scandirat(int dir_fd, const char *dir, struct dirent ***dirlist,
 	int (*filter)(const struct dirent *),
 	int (*compar)(const struct dirent **, const struct dirent **))
@@ -57,7 +64,8 @@ scandirat(int dir_fd, const char *dir, struct dirent ***dirlist,
 	}
 	*dirlist = ret;
 
-	qsort(ret, retlen, sizeof(*ret), (void *)compar);
+	if (compar != NULL)
+		qsort(ret, retlen, sizeof(*ret), (void *)compar);
 
 	/* closes underlying fd */
 	closedir(dirp);
@@ -67,7 +75,7 @@ scandirat(int dir_fd, const char *dir, struct dirent ***dirlist,
 
 #endif
 
-static void
+void
 scandir_free(struct dirent **de, int cnt)
 {
 	if (cnt <= 0)
@@ -77,3 +85,23 @@ scandir_free(struct dirent **de, int cnt)
 		free(de[cnt]);
 	free(de);
 }
+
+int
+filter_hidden(const struct dirent *de)
+{
+	if (de->d_name[0] == '.')
+		return 0;
+	return 1;
+}
+
+int
+filter_self_parent(const struct dirent *de)
+{
+	if (de->d_name[0] == '.' &&
+		(de->d_name[1] == '\0' ||
+		 (de->d_name[1] == '.' &&
+		  de->d_name[2] == '\0')))
+		return 0;
+
+	return 1;
+}
